/* 
 * File:   I2CTest.c
 * Author: Bob Kondner
 *
 * Created on November 15, 2014, 2:23 PM
 *
 * Example of using a SunDuino to control a 4 digit 7 Segment LED display
 * via I2C Target uses a NXP (aka Philips) SAA1064 I2C controller.
 *
 * Includes status display on serial port and implements a few commands
 * in SerialStatus.c.
 */

#include <stdio.h>
#include <stdlib.h>

#include <xc.h>
#include "../SunDuino_V100.h"

#include "plib/i2c.h"

#include "SerialStatus.h"

/*
 * We have a bunch of stuff to define the 7 Segment Character control image.
 */

#define seg_dp		0b10000000			// RD0
#define seg_a		0b01000000			// RD1
#define seg_b		0b00100000			// RD2
#define seg_c		0b00010000			// RD3
#define seg_d		0b00001000			// RD4
#define seg_e		0b00000100			// RD5
#define seg_f		0b00000010			// RD6
#define seg_g		0b00000001			// RD7

// Mask for all of the Segment Select Lines
#define	SEG_MASK (seg_a | seg_b | seg_c | seg_d | seg_e | seg_f | seg_g | seg_dp)


// end of Hardware Definitions
//-------------------------------------------------------------------------------------------------
// beginning of Constant Definitions


// Definitions of each character
// For a particular character, define which segments should
// be lit.

#define dark		(~SEG_MASK)			// No Segments Lit
#define char_0		(seg_a | seg_b | seg_c | seg_d | seg_e | seg_f)	// 0
#define char_1		(seg_b | seg_c)					// 1
#define char_2		(seg_a | seg_b | seg_g | seg_e | seg_d)		// 2
#define char_3		(seg_a | seg_b | seg_g | seg_c | seg_d)		// 3
#define char_4		(seg_f | seg_g | seg_b | seg_c)			// 4
#define char_5		(seg_a | seg_f | seg_g | seg_c | seg_d)		// 5
#define char_6		(seg_a | seg_f | seg_g | seg_e | seg_c | seg_d)	// 6
#define char_7		(seg_a | seg_b | seg_c)				// 7
#define char_8		(seg_a | seg_b | seg_c | seg_d | seg_e | seg_f | seg_g)	// 8
#define char_9		(seg_a | seg_b | seg_f | seg_g | seg_c)		// 9

// Character Generator Array
// These values designate which segments should be lit for any given character
// The index is the digit value for characters 0 - 9
// for example:  segment[4] gives the proper segments to light up a character 4.
// other characters follow, for example hyphen, and various other symbols

const unsigned char seg_sel[] = {
		char_0,		// 0
		char_1,		// 1
		char_2,		// 2
		char_3,		// 3
		char_4,		// 4
		char_5,		// 5
		char_6,		// 6
		char_7,		// 7
		char_8,		// 8
		char_9,		// 9
		dark,		// 10
		};


signed char status;

extern void ShowSerialStatus (unsigned char SleepRes, unsigned char DidSleep);


void I2CByteWrite (unsigned char mybyte) {
    if (status == 0) status = WriteI2C(mybyte);
 }


void main (void)
{
    unsigned char OldSec;
    unsigned char Sec;
    unsigned int  Value;
    unsigned char doblank;
    unsigned int  AutoOff;
    unsigned char WasOff;
    unsigned char dplocation;
    unsigned char mydp;

    struct TPowerEvents     PowerEvents;
    struct TPowerState      PowerState;
    
    RX_ProcessInit();       //Clear RX Character processing logic

    PowerEvents = bb_getPowerEvents();  //Clear any old Events like Low Battery
    bb_func_ClearEvents(PowerEvents);
    
    bb_func_PwrOnOff(1);                //Turn Power Converter On

    OldSec = 255;                       //Causes quick first loop into code
    AutoOff = 10;                       //Start with 10 Seconds On

    while ( 1 ) {

        do {
            RX_Process();               //Process any RX Chars as they prevent sleep
            bb_func_DoSleep ();

/*
 *  Here we enter into a loop where we wait for 1 seconds in Sleep Mode
 */
            Sec = bb_time_getSeconds();
        } while ( Sec == OldSec);

        OldSec = Sec;                       //Setup for next 1 second loop.
        RX_ProcessTime ();                  //Every second do TX Timeout Logic

        TRISA6 = 0;                         //Keep bit an output
        PORTAbits.RA6 = Sec & 0x01;         //Flip RS6 every second

        if ( (Sec % 10) == 4) ShowSerialStatus(0,0);

/*
 * Drop Through Means 1 second has expired so we need to process
 * our Display Update if power is ON. If not ON we reset I2C output bits.
 *
 * If we see the input voltage as applied (ChargerOK) and the power is OFF
 * then we force Power back ON.
 */
        if (bb_getPowerState().PS_ChargerOK && (! bb_func_PowerIsOn()) )
        {
            bb_func_PwrOnOff(1);
            delay_ms(20);           //Give power a chace to come up
        }

        if (bb_func_PowerIsOn() )
        {
            if (WasOff) AutoOff = 10;   //If power went ON reset 10 second timer.
            WasOff = 0;
 
 /*
  * Power On Loop
  *
  * Power is applied to the LED display so we need to update the LEDs with data.
  * Since Power might have been off we reconfigure the PORTC I2C bits.
  *
  */
            TRISC3  = 1;
            TRISC4  = 1;
            SSPADD = 40;//   0x0A;                //SCL Clock Period

            status = 0;
            OpenI2C (MASTER, SLEW_OFF);

            IdleI2C ();
            StartI2C();

            I2CByteWrite (0x70);       //Address of SAA1064 Control Reg
            I2CByteWrite (0x0);        //Control Reg Address
            I2CByteWrite (0x27);       //Control Reg Settings

/*
 * Here we have some options examples of display a value onto the LED Digits
*/
            if (Sec & 0x2)
            {
                Value = bb_battery_getGas();
                dplocation = 0;
            }
            else
            {
                Value =  (int)(bb_volts_getVUser() * 200.0);  //Scale to 1000 from the default 10V
                dplocation = 3;
            }

            if (Value > 9999) Value = 9999;
            doblank = 1;

            if (dplocation > 3) doblank = 0;
            if ( ((Value / 1000) % 10) == 0 ) {
                I2CByteWrite (0);
            } else {
                if (dplocation == 4) mydp = seg_dp; else mydp = 0;
                I2CByteWrite (mydp | seg_sel[ (Value / 1000) % 10 ] ); // Set all 4 Digits;
                doblank = 0;
            }

            if (dplocation > 2) doblank = 0;
            if ( doblank && ((Value / 100) % 10) == 0 ) {
                I2CByteWrite (0);
            } else {
                if (dplocation == 3) mydp = seg_dp; else mydp = 0;
                I2CByteWrite (mydp | seg_sel[ (Value / 100) % 10 ] ); // Set all 4 Digits;
                doblank = 0;
            }

            if (dplocation > 1) doblank = 0;
            if ( doblank && ((Value / 10) % 10) == 0 ) {
                I2CByteWrite (0);
            } else {
                if (dplocation == 2) mydp = seg_dp; else mydp = 0;
                I2CByteWrite (mydp | seg_sel[ (Value / 10) % 10 ] ); // Set all 4 Digits;
                doblank = 0;
            }


            if (dplocation == 1) mydp = seg_dp; else mydp = 0;
            I2CByteWrite (mydp | seg_sel[ (Value) % 10 ] ); // No blank last digit

            CloseI2C();

/*
 * We just did a display update while Power is On. We need to Bump Down the AutoOff
 * timer and if 0 turn off the power.
 */
            if (AutoOff == 0) {
                bb_func_PwrOnOff(0);
                AutoOff = 600;          //Setup for 10 Minutes off
            }
            else
            {
                if (bb_getBatteryStatus().BS_OnBattery)   AutoOff--;
            }

        }
/*
 * Power Off Loop
 *
 * At ELSE power is off so force I2C pins to 0 as the load is powered off.
 * Keeping pins high (Normal I2C conditions) would drive current into the
 * I2C pullups and draw current from the battery during sleep mode.
 * We also keep the AutoOff setup for when the power comes on again
 */
        else {
            SSPEN = 0;                  //Shut down I2C operation
            TRISC3  = 0;
            TRISC4  = 0;
            PORTCbits.RC4 =  0;
            PORTCbits.RC3 =  0;

/*
 *  As long as the battery is NOT failed we turn On once every 10 minutes.
 * This time of 600 seconds is set above in the PowerOn loop.
 * Using PwrOnOff(1) is dangerious if the Battery is Failed. Forcing ON
 * when failed will clear the failed flag. Another failed will be issued but
 * constantly forcing on will take the battery very low. Not Good.
 */
            WasOff = 1;             //Signal power was off
            
            if (!bb_getBatteryStatus().BS_Fail)
            {
                if (AutoOff == 0)
                {
                    bb_func_PwrOnOff(1);
                    AutoOff = 10;       //Setup for 10 seconds on
                } else AutoOff--;

            } else AutoOff = 10;        //Turn On 10 Seconds after FAIL clears.

/*
 * Watch if the Battery starts charging. If so reduce the timer to 1 seconds.
 * We must wait for a Charge Request and a ChargerOK. This will only trigger
 * while the charging process is active. The FAIL flag, if active, must clear
 * (above) before power is switched On.
 */
            PowerState = bb_getPowerState();
            if ( PowerState.PS_ChargeRequired
                 & PowerState.PS_ChargerOK
                 & (AutoOff > 1)
               ) AutoOff = 1;
        }
    }

}

