/*
 * The is a module that implemets SunDuino status and control using the
 * RS-232 port at 9600 baud (Default Setting)
 *
 * A small command line parser takes in characters and decodes a few commands.
 *
 * Bob Kondner November 16, 2014
 * 
 */

#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>


#include "../SunDuino_V100.h"
#include "SerialStatus.h"

static char RX_Command[MAXCMD];
static char RX_Token[MAXTKN];
static char RX_Timer;              //Used to timer out Rx Command Strings


void RX_Execute(void); // Call to decode and execute command in RX_Command

void ShowSerialStatus(unsigned char SleepRes, unsigned char DidSleep) {
    float VRun, VBat, VCharger, VUser, IAvg;
    unsigned char Hours, Min, Sec, Year, Month, Date;
    struct TPowerEvents PowerEvents;
    struct TPowerState PowerState;
    struct TBatteryStatus BatteryStatus;

// If we are in the middle of a command line do not write to the UART

    if (RX_Command[0] == 0) {       //0 First Char means no command line

        Hours = bb_time_getHours();
        Min   = bb_time_getMinutes();
        Sec   = bb_time_getSeconds();
        Year  = bb_date_getYear();
        Month = bb_date_getMonth();
        Date  = bb_date_getDate();

        bb_func_WDT(RX_TimeOut + 25); //Allow time to RX TimeOuts
        VRun        = bb_volts_getVRun();
        VBat        = bb_volts_getVBattery();
        VCharger    = bb_volts_getVCharger();
        VUser       = bb_volts_getVUser();
        IAvg        = bb_amps_getAverageCurrent();

        printf("\r\nDate\\Time  20%02d/%02d/%02d   %02d:%02d:%02d\r\n",
               Year,Month,Date, Hours, Min, Sec);

        printf("VRun  = %5.2f  VBat= %5.2f  VChrg = %5.2f  VUser = %5.2f\r\n",
                VRun, VBat, VCharger, VUser);

        printf("ILoad = %5.2f Amp  Cap = %ld of %d  %3d%%  Bat Temp = %d\r\n",
                IAvg,
                bb_battery_getGas(),
                ((int) bb_battery_getSize()),
                ((int) bb_battery_getPercentage()),
                bb_battery_getTemperature());

        printf("Partial = %d   Reserve = %d ",
                bb_battery_getPartialCharge(),
                bb_battery_getReserveCharge());

        // Print is SLEEP was ever entered and then Print No Sleep Reurn Values
        if (DidSleep)
            printf("  Did Sleep");
        else printf("   No Sleep");

        if (SleepRes > 0) printf(":");
        if (SleepRes == 1) printf(" Analog");
        if (SleepRes == 2) printf(" No Sleep Timer");
        if (SleepRes == 3) printf(" Not Idle State");
        if (SleepRes == 4) printf(" Power Error");
        if (SleepRes == 5) printf(" Bad EEPROM Checksum");
        if (SleepRes == 6) printf(" Missing Battery");
        if (SleepRes == 7) printf(" UART RX Busy");
        if (SleepRes == 8) printf(" UART TX Char Waiting");
        if (SleepRes == 9) printf(" UART TX Shifting");

        PowerEvents = bb_getPowerEvents();
        printf("\r\nPower Events\r\n");
        if (PowerEvents.PE_Reset) printf("   Reset\r\n");
        if (PowerEvents.PE_Button) printf("   Button Pushed\r\n");
        if (PowerEvents.PE_Missingbat) printf("   Missing Battery\r\n");
        if (PowerEvents.PE_PwrError) printf("   Power Error\r\n");
        if (PowerEvents.PE_WDTEspire) printf("   WDT Expire\r\n");
        if (PowerEvents.PE_Alarm) printf("   Alarm\r\n");
        /*
         * Since some Power Events will prevent the processor from sleaping we now
         * clear any Power Events we read. By only cleaing what we just read we will
         * not accidently clear any we have not yet read. (IE something that just happened!)
         */
        bb_func_ClearEvents(PowerEvents);

        PowerState = bb_getPowerState();
        printf("\r\nPower State\r\n");
        if (PowerState.PS_ChargeEn) printf("   Charge Enabled\r\n");
        if (PowerState.PS_ChargeRequired) printf("   Charge Required\r\n");
        if (PowerState.PS_ChargerOK) printf("   Charger OK\r\n");
        if (PowerState.PS_ExternButton) printf("   External Button\r\n");
        if (PowerState.PS_HostPwrHold) printf("   Power Toggle Mode\r\n");
        if (PowerState.PS_HostPwrLock) printf("   Power Locked\r\n");
        if (PowerState.PS_PwrReqest) printf("   Power Request\r\n");
        if (PowerState.PS_TopOff) printf("   Top Off Active\r\n");

        BatteryStatus = bb_getBatteryStatus();
        printf("\r\nBattery Status\r\n");
        if (BatteryStatus.BS_Fail) printf("   Battery Failed\r\n");
        if (BatteryStatus.BS_Full) printf("   Battery Full\r\n");
        if (BatteryStatus.BS_Low) printf("   Battery Under Reserve \r\n");
        if (BatteryStatus.BS_New) printf("   New Battery\r\n");
        if (BatteryStatus.BS_OnBattery) printf("   Running On Battery\r\n");
        if (BatteryStatus.BS_TopOffEnabled) printf("   Top Off Enabled\r\n");
        if (BatteryStatus.BS_VoltageLow) printf("   Battery Voltage low\r\n");
        if (BatteryStatus.BS_Warning) printf("   Battery Level Near Fail\r\n");
        printf("\r\n\r\nBB>");

    }
}

/*
 * Below is the code used to process incoming UART character. These routines are
 * called from a couple different locations so that collection of user entered
 * command strings takes place as a background task.
 *
 * RX_ProcessInit   Clear variable and restarts command collection
 * RX_Process       Called on exit from DoSleep, takes chars from UART
 * RX_TimeOut       Called once a second to timeout unfinished commands
 * RX_Execute       Called when a CR is received. Decodes commands
 *
 */
void RX_ProcessInit(void) {
    char i;
    while (RCIF) i = RCREG;
    for (i = 0; i < MAXCMD; i++) RX_Command[i] = 0; //Prefill cmd string with 0
    RX_Timer = 0;
}

/*
 * We pull any chars out of the RX UART and place them into a RX_Command string.
 * When we get a CR we try to execute the command. Execute finishes by clearing
 * RX_Command which starts the collection of a new string.
 */
void RX_Process(void) {
    char InChar;
    char NextPos;
    if (RCIF) {
        RX_Timer = 0;
        InChar = RCREG; //Read char will allow Sleep
        NextPos = strlen(RX_Command);
        if (InChar == 0x0d) InChar = 0x0a;
        if (InChar == 0x08) //Deal with Backspace
        {
            printf("\b \b");
            if (NextPos > 0) {
                RX_Command[NextPos - 1] = 0;
            }
        } else {
            if (NextPos < (MAXCMD - 1)) //If cmd string not full add new char
            {
                RX_Command[NextPos] = InChar;
            }
            if (InChar == 0x0a) {
                printf("\r\n");
                RX_Execute();
            } else printf("%c", InChar);
        }
    }
}

// RX_ProcessTime is called once a second and it times the timeout period.

void RX_ProcessTime(void) {
    if (RX_Timer++ >= RX_TimeOut) {
        RX_ProcessInit();
        printf("\r\nRX Timeout\r\n");
    } else if (RX_Command[0] == 0) RX_Timer = 0;

}

char RX_GetToken(char Pntr) {
    char i;
    for (i = 0; i < MAXTKN; i++) RX_Token[i] = 0;
    i = 0;
    while ((Pntr < MAXCMD) & (RX_Command[Pntr] <= 0x20)) Pntr++;
    while ((Pntr < MAXCMD) & (RX_Command[Pntr] > 0x20)) {
        if (i < (MAXTKN - 1)) {
            RX_Token[i] = RX_Command[Pntr];
            i++;
        }
        Pntr++;
    }
    return (Pntr);
}

// Attempt to decode and execute a complete command

void RX_Execute(void) {
    char InCntr;
    char i;
    unsigned long LongVar;

    for (i = 0; i < MAXCMD; i++) {
        if ((RX_Command[i] >= 'a') & (RX_Command[i] <= 'z'))
            RX_Command[i] -= 0x20;
        if (RX_Command[i] == 0x09) RX_Command[i] = 0x20;
    }

    //    InCntr = _doscan (RX_Command,"%s %d",RX_Token,&LongVar);
    InCntr = 0;
    InCntr = RX_GetToken(InCntr);

    if (strcmp("GAS", RX_Token) == 0) {
        InCntr = RX_GetToken(InCntr);
        LongVar = strtol(RX_Token, 0, 10);

        printf("   Gas Now: %d \r\n", LongVar);
        bb_battery_setGas(LongVar);
        RX_Command[0] = 0;
    }

    if (strcmp("ON", RX_Token) == 0) {
        bb_func_PwrOnOff(1);
        printf("   Power Locked On \r\n");
        RX_Command[0] = 0;
    }

    if (strcmp("OFF", RX_Token) == 0) {
        bb_func_PwrOnOff(0);
        printf("   Power Locked Off \r\n");
        RX_Command[0] = 0;
    }

    if (strcmp("TIME", RX_Token) == 0) {
        unsigned char inhour;
        unsigned char inmin;
        unsigned char insec;
        InCntr = RX_GetToken(InCntr);
        inhour = strtol (RX_Token, 0, 10);
        InCntr = RX_GetToken(InCntr);
        inmin  = strtol (RX_Token, 0, 10);
        InCntr = RX_GetToken(InCntr);
        insec  = strtol (RX_Token, 0, 10);
        bb_time_set (inhour, inmin, insec);
        RX_Command[0] = 0;
    }

    if (strcmp("DATE", RX_Token) == 0) {
        unsigned char inyear;
        unsigned char inmonth;
        unsigned char indate;
        unsigned char inday;
        InCntr  = RX_GetToken(InCntr);
        inyear  = strtol (RX_Token, 0, 10);
        InCntr  = RX_GetToken(InCntr);
        inmonth  = strtol (RX_Token, 0, 10);
        InCntr  = RX_GetToken(InCntr);
        indate  = strtol (RX_Token, 0, 10);
        InCntr  = RX_GetToken(InCntr);
        inday   = strtol (RX_Token, 0, 10);
        bb_date_set (inyear, inmonth, indate, inday);
        RX_Command[0] = 0;
    }

    if (strcmp("?", RX_Token) == 0) {
        printf("   On        Turn On Output Power\n\r");
        printf("   Off       Turn Off Output Power\n\r");
        printf("   Gas n     Set Battery Capacity (mahr) to n (0 - 64000\n\r");
        printf("   TopOff n  Top Off Battery for n Minutes (1 - 255)\n\r");
        printf("   Date yy mm dd dw (0-99 1-12 1-31 1-7)\n\r");
        printf("   Time hh mm ss    (0-23 0-59 0-59)\n\r");
        RX_Command[0] = 0;
    }

    if (strcmp("TOPOFF", RX_Token) == 0) {
        InCntr = RX_GetToken(InCntr);
        LongVar = strtol(RX_Token, 0, 10);
        bb_func_TopOff((int) LongVar);
        printf("   Top Off set to: %d Minutes\r\n", LongVar);
        RX_Command[0] = 0;
    }

    if ((RX_Command[0] != 0) & (RX_Command[0] != 0x0a))
        printf("\n\rBad Command: %s\r\n\r\nBB>", RX_Command);
    else printf("\n\rBB>");

    RX_ProcessInit();


    return;

}
