/* 
 * File:   BB_Functions.h
 * Author: pwebber anr rkondner
 *
 * Created on January 17, 2014, 1:24 PM
 * Updated on Novenber 29. 2014
 */

#ifndef SUNDUINO_V100_H
#define	SUNDUINO_V100_H

/*
 * There are 3 major bytes in low level SunDuino code used to control various
 * state machines. Power Event bits are typical set by various events and are
 * then cleared by user applications as they are read. Power State is used for
 * power On/Off and charger control. Note that charging logic is complex as a
 * software PWM is used to control charging and various events can disable
 * charging. Charge Required is the main bit that indicates when the battery
 * required sharging. TopOff is the process by which a battery is slightly
 * over charged.
 *
 * Battery Status contains a number of useful bits about battery state.
 */

struct TPowerEvents {
    unsigned PE_Reset        : 1;   //Indicates the Processor has been reset
    unsigned PE_PwrError     : 1;   //Indicates there has been a Power Error
    unsigned PE_Missingbat   : 1;   //Indicates the main battery has is not connected
    unsigned PE_Alarm        : 1;   //Indicates the Alamr Timer has expires
    unsigned PE_Button       : 1;   //Indicates the external button has turned on Power
    unsigned PE_NotUsed5     : 1;
    unsigned PE_WDTEspire    : 1;   //Indicates the low level WDT timer has expired.
    unsigned PE_NotUsed7     : 1;
};

struct TPowerState {
    unsigned PS_PwrReqest    : 1;   //Indicates something is requesting a power On
    unsigned PS_ChargeEn     : 1;   //Used to turn On or Off the Charging Circuit
    unsigned PS_ExternButton : 1;   //State of the external ON/OFF signal
    unsigned PS_HostPwrLock  : 1;   //Indicates Power is Locked On
    unsigned PS_HostPwrHold  : 1;   //Indicates ON Signal is a toggle On/Off
    unsigned PS_ChargerOK    : 1;   //Indicates the input voltage is ready for charging
    unsigned PS_ChargeRequired : 1; //Indicates that battery charging is required
    unsigned PS_TopOff       : 1;   //Indicate that the Top Off Timer is running
};

struct TBatteryStatus {
    unsigned BS_OnBattery    : 1;   //Indicates input low, running on Battery
    unsigned BS_Full         : 1;   //Indicate battery is completely charged
    unsigned BS_Fail         : 1;   //Indicates the battery has been completely discharged
    unsigned BS_Warning      : 1;   //Indicates battery voltage is very low, about to shut down
    unsigned BS_New          : 1;   //Indicates possible new battery.
    unsigned BS_Low          : 1;   //Indicates the battery is below the Battery Reserve level
    unsigned BS_TopOffEnabled : 1;  //Indicates the Top Off charging process is enabled
    unsigned BS_VoltageLow   : 1;   //Indicates the battery Voltage is Low
};

/*
 * And we have three functions for reading these structures
 */

struct TPowerEvents     bb_getPowerEvents (void);
struct TPowerState      bb_getPowerState (void);
struct TBatteryStatus   bb_getBatteryStatus (void);

/*
 * SunDuino monitoers battery temperature from -40C to 80C.
 */
int bb_battery_getTemperature();

/*
 * We have several routines that use longs to handle battery capacity in mahr.
 * The Gas is the current batter capacity. Size is the batter capacity rating.
 * Partial is used internally in the charging process, very important but not
 * really useful to the user application. Reserve is a trigger point for shutting
 * down the DC output regulator. Some PC systems take a long time to boot and
 * you do not want to abort the boot process. Reserve setting help ensure there
 * is a minimal amount of battery power in reserve.
 */
unsigned long bb_battery_getGas();                      /* Done */
void          bb_battery_setGas(unsigned long NewGas);  /* Done */
unsigned long bb_battery_getSize();                     /* Done */
unsigned long bb_battery_getPartialCharge();            /* Done */
unsigned long bb_battery_getReserveCharge();            /* Done */
char          bb_battery_getPercentage();               /* Done */

/*
 * The SunDuino has three voltages and one current that are very important to
 * system operation. VRun is typically a diode drop below the DC input voltage.
 * Since the SunDuino uses a simple constant current PWM charger the DC input
 * voltage must be at least 1 volt above the battery voltage. The difference
 * is called the Charger voltage. Voltages are from 0 to 40V with a step size of
 * about 160mv. Voltages are about 2% +/- 1 step size.
 *
 * The Current is the current being used by the DC output regulator. The
 * total current drawn from the DC input is this regulator current plus any
 * battery charging current. Current is in Amps and can range from 0 to 6.6 Amps.
 */
float bb_volts_getVRun();                               /* Done */
float bb_volts_getVBattery();
float bb_volts_getVCharger();
float bb_volts_getVUser();          // Typically 10.0V Full Scale

float bb_amps_getAverageCurrent();

/*
 * There are a couple of functions used to comitor and control operation.
 *
 * DoSleep is used to permit the SunDuino to enter sleep is conditions allow. The
 * returned result give a indication that the processor slept or an indication of
 * what prevented sleep.
 * PowerOnOff will turn the DC output regulator ON.
 * WDT allows the user application to set a time to reset to clear a hung app.
 * ClearEvents is used to clear Power Events once they are detected and processed
 * PwrLock allows the user app to prevent a user from turning off power during data backup.
 */

unsigned char bb_func_DoSleep(void);                    /* Done */
void bb_func_PwrOnOff(char myOnOff);                    /* Done */
void bb_func_WDT(char TimeOutSec);
void bb_func_TopOff (int TopOffMin);
void bb_func_ClearEvents (struct TPowerEvents MyEvents);
void bb_func_PwrLock (char Lock);
unsigned char bb_func_PowerIsOn (void);

/*
 * The low level Sunduino code maintains a Real Time Clock and these routines
 * allow the user app to read these times and dates.
 */
unsigned char bb_time_getSeconds();
unsigned char bb_time_getMinutes();
unsigned char bb_time_getHours();
void          bb_time_set (unsigned char inHours, unsigned char inMinutes, unsigned char inSeconds);

unsigned char bb_date_getDay();
unsigned char bb_date_getDate();
unsigned char bb_date_getMonth();
unsigned char bb_date_getYear();
void          bb_date_set (unsigned char inYear, unsigned char inMonth,
                           unsigned char inDate, unsigned char inDay);

/*
 * Se have a couple routines to keep the C runtime happy
 */
void putch(char c);                                     /* Done */
void delay_ms(int mytime);                              /* Done */

#endif	/* SUNDUINO_V100_H */

